
/*--------------------------------------
FILE NAME       : can841.C
VERSION         : V1.0
--------------------------------------*/
#include "can841.h"
#include <dos.h>


#define MK_FP(seg,off) \
        ((void far *)(((unsigned long)(seg)<<16) | (unsigned)(off)))


#define IRQ_POLLING     0
#define NULL            0
/*--------------------------------------------------------------------------*/
BYTE gIrq0, gIrq1;
BYTE old_irq0=IRQ_POLLING;
BYTE old_irq1=IRQ_POLLING;
UI   gSeg;
int hw_ok=0;



void ( interrupt far *old_isr0)();
void ( interrupt far *old_isr1)();




int can_err_no;
        /*------------------------------------------------
        0: no error
        0x01 : hardware not release.
        0x10: seg_addr set error
        0x11: hardware not set successful
        0x12: irq_no set error
        0x20: bus off
        ------------------------------------------------*/
/*--------------------------------------------------------------------------*/
void can0w(BYTE addr, BYTE v);
void can1w(BYTE addr, BYTE v);
BYTE can0r( BYTE addr);
BYTE can1r( BYTE addr);
void  can_reset( BYTE port );



/*--------------------------------------------------------------------------*/
BYTE rBuf0[10], rBuf1[10];
BYTE gSendFlag0, gSendFlag1;
BYTE gRecFlag0, gRecFlag1;
/*--------------------------------------------------------------------------*/
void  interrupt far can_isr0()
{
  unsigned char flag;
  int i;

  _disable();
  flag=can0r(3);
  if( flag & 1) /* RIF */
  {
    rBuf0[0]=can0r(20);
    rBuf0[1]=can0r(21);
    for(i=0; i< (rBuf0[1]&0x0f); i++)
      rBuf0[i+2]=can0r((BYTE)(22+i));
    /* release receive buffer*/
    can0w(1,4);
    gRecFlag0=1;
  }
  if( flag & 2) /* TIF */
  {
    gSendFlag0=1;
  }
   outp(0x20,0x20);
   outp(0xA0,0x20);
   _enable();
}
/*--------------------------------------------------------------------------*/
void  interrupt far can_isr1()
{
  unsigned char flag;
  int i;

  _disable();
  flag=can1r(3);
  if( flag & 1) /* RIF */
  {
    rBuf1[0]=can1r(20);
    rBuf1[1]=can1r(21);
    for(i=0; i< (rBuf1[1]&0x0f); i++)
      rBuf1[i+2]=can1r((BYTE)(22+i));
    /* release receive buffer*/
    can1w(1,4);
    gRecFlag1=1;
  }
  if( flag & 2) /* TIF */
  {
    gSendFlag1=1;
  }
   outp(0x20,0x20);
   outp(0xA0,0x20);
   _enable();
}
/*--------------------------------------------------------------------------*/
void getIntNo(BYTE irq_no, BYTE * int_no)
{
  BYTE int_tbl[16]={
  0xff,0xff,0xff,0x0b,0x0c,0x0d,0x0e,0x0f,
  0xff,0x71,0x72,0x73,0x74,0xff,0xff,0x77
  };
  *int_no = int_tbl[irq_no];
}
/*--------------------------------------------------------------------------*/
int canInitHW( UI segment, BYTE irq0, BYTE irq1)
{
  BYTE data;
  BYTE int_no;

  /*-----------------------
  If app. call this function again, error it.
  --------------------------*/
  /*-----------------------
  if( hw_ok!=0)
  {
    can_err_no=0x1;
    return(0);
  }
  -------------------------*/
  if( segment >0xdf00 || segment <0xc000 ||(segment%0x100))
  {
    can_err_no=0x10;
    return(0);
  }
  gSeg=segment;
  data=can0r(0);
  if( data &0x01)
  {
    can0w(0,0x60);
    if(can0r(0) &0x01)
    {
      can_err_no=0x10;
      return(0);
    }
  }
  can_reset(0);
  if(!(can0r(0) &0x01))
  {
      can_err_no=0x10;
      return(0);
  }
  /*----- irq_set  error check. ------------------*/
  if( irq0 != 0 && irq0 != 3 && irq0 != 4 && irq0 !=5 && irq0 !=6 && irq0 !=7
      && irq0 !=9 &&  irq0 !=10 && irq0 !=11 && irq0 !=12 && irq0 != 15)
  {
      can_err_no=0x12;
      return(0);
  }
  if( irq1 != 0 && irq1 != 3 && irq1 != 4 && irq1 !=5 && irq1 !=6 && irq1 !=7
      && irq1 !=9 && irq1 !=10 && irq1 !=11 && irq1 !=12 && irq1 != 15)
  {
      can_err_no=0x12;
      return(0);
  }
  if( irq0 == irq1  && irq0 !=IRQ_POLLING)
  {
      can_err_no=0x12;
      return(0);
  }
  getIntNo(irq0, &data);
  if( data == 0xff && irq0 !=0)
  {
      can_err_no=0x12;
      return(0);
  }
  getIntNo(irq1, &data);
  if( data == 0xff && irq1 !=0)
  {
      can_err_no=0x12;
      return(0);
  }/*..... irq_set  error check. ..................*/

  gIrq0=irq0;
  if( gIrq0 != old_irq0)
  {
    /*---- restore old irq int service routine -------*/
    if( old_irq0 !=IRQ_POLLING)
    {
      data=inp(0x21);
      outp(0x21,0xff);
      _disable();
      getIntNo(old_irq0, &int_no);
      _dos_setvect(int_no,old_isr0);

      _enable();
      outp(0x21,data);

      old_irq0 = IRQ_POLLING;
    }/*.... restore old irq int service routine .......*/

    /*---- set new irq int service routine -------*/
    if( gIrq0 != IRQ_POLLING) /* not polling */
    {
      getIntNo(gIrq0,&int_no);
      old_isr0=_dos_getvect(int_no);
      old_irq0 = gIrq0;

      data=inp(0x21);
      outp(0x21,0xff);
      outp(0xA1,0xff);
      _disable();
      getIntNo(gIrq0,&int_no);
      _dos_setvect(int_no,can_isr0);

      _enable();
      outp(0x21,0x00);
      outp(0xA1,0x01);
    }
  }

  gIrq1=irq1;
  if( gIrq1 != old_irq1)
  {
    /*---- restore old irq int service routine -------*/
    if( old_irq1 !=IRQ_POLLING)
    {
      data=inp(0x21);
      outp(0x21,0xff);
      _disable();
      getIntNo(old_irq1, &int_no);
      _dos_setvect(int_no,old_isr1);

      _enable();
      outp(0x21,data);

      old_irq1 = IRQ_POLLING;
    }/*.... restore old irq int service routine .......*/

    /*---- set new irq int service routine -------*/
    if( gIrq1 != IRQ_POLLING) /* not polling */
    {
      getIntNo(gIrq1, &int_no);
      old_isr1=_dos_getvect(int_no);
      old_irq1 = gIrq1;

      data=inp(0x21);
      outp(0x21,0xff);
      outp(0xA1,0xff);
      _disable();

      getIntNo(gIrq1, &int_no);
      _dos_setvect(int_no,can_isr1);

      _enable();
      outp(0x21,0x00);
      outp(0xA1,0x01);
    }
  }

  hw_ok=1;
  return(1);
}
/*--------------------------------------------------------------------------*/
void  can_reset( BYTE port )
{

        BYTE huge *ptr=MK_FP(gSeg+0x200*port,0x100);
        BYTE a;
        int i,j;

        a=*ptr;
        *ptr=a;
        for( i=0;i<100; i++)
          for(j=0;j<1000; j++);
}
/*--------------------------------------------------------------------------*/
void can0w(BYTE addr, BYTE v)
{
        BYTE huge *ptr;
        ptr = MK_FP( gSeg,addr);
        *ptr=v;
}
/*--------------------------------------------------------------------------*/
void can1w(BYTE addr, BYTE v)
{
        BYTE huge *ptr;
        ptr = MK_FP( gSeg,addr+0x200);
        *ptr=v;
}
/*--------------------------------------------------------------------------*/
BYTE can0r( BYTE addr)
{
  BYTE v;
  BYTE huge *ptr;
        ptr = MK_FP(gSeg,addr);
        v=*ptr;
        return(v);
}
/*--------------------------------------------------------------------------*/
BYTE can1r( BYTE addr)
{
  BYTE v;
  BYTE huge *ptr;
        ptr = MK_FP(gSeg,addr+0x200);
        v=*ptr;
        return(v);
}
/*--------------------------------------------------------------------------*/
BYTE canExitHW()
{
  unsigned char d=inp(0x21);
  BYTE int_no;
  /* reset old isr */
  outp(0x21,0xff);
  _disable();
  if( old_isr0 != NULL )
  {
       getIntNo(old_irq0,&int_no);
       _dos_setvect(int_no,old_isr0);
       old_irq0=IRQ_POLLING;
  }
  if( old_isr1 != NULL )
  {
       getIntNo(old_irq0,&int_no);
       _dos_setvect(old_irq1,old_isr1);
       old_irq1=IRQ_POLLING;
  }
  _enable();
  outp(0x21,d);
}
/*--------------------------------------------------------------------------*/
int  canReset( BYTE port )
{
  can_reset( port );
}
/*--------------------------------------------------------------------------*/
int  canConfig( BYTE port, CAN_STRUCT can)
{
  BYTE temp;
  canReset( port );
  if( port ==0)
  {
    if( gIrq0 == IRQ_POLLING)
      can0w(0,1);
    else
     can0w(0,3);/* reset mode and receive irq */
  }
  else
  {
    if( gIrq1 == IRQ_POLLING)
      can1w(0,1);
    else
     can1w(0,3);/* reset mode and receive irq */
  }
  if( port ==0)
  {
     can0w(6,can.bt0); /* BT0 */
     can0w(7,can.bt1); /* BT1 */
     temp=can0r(6);
     if( temp !=can.bt0)
       return(0);
     temp=can0r(7);
     if( temp !=can.bt1)
       return(0);
  }
  else
  {
     can1w(6,can.bt0); /* BT0 */
     can1w(7,can.bt1); /* BT1 */
     temp=can1r(6);
     if( temp !=can.bt0)
       return(0);
     temp=can1r(7);
     if( temp !=can.bt1)
       return(0);
  }
   if( port ==0)
   {
     can0w(4,can.acc_code);/* accept code */
     can0w(5,can.acc_mask);/* accept code */
   }
   else
   {
     can1w(4,can.acc_code);/* accept code */
     can1w(5,can.acc_mask);/* accept code */
   }
   if( port ==0)
   {
     can0w(8,0xfa);
   }
   else
   {
     can1w(8,0xfa);
   }
  return(1);
}
/*--------------------------------------------------------------------------*/
int  canNormalRun( BYTE port )
{
BYTE v;
  if( port ==0)
  {
        v=can0r(0);
        if( v & 1)
          can0w(0,(BYTE)(v&0xfe));
  }
  else
  {
        v=can1r(0);
        if( v & 1)
          can1w(0,(BYTE)(v&0xfe));
  }
}
/*--------------------------------------------------------------------------*/
int canSendMsg( BYTE  port, MSG_STRUCT msg)
{
  BYTE v;
  BYTE i;

  if( port==0)
  {
    v=msg.id>>3;
    can0w(10,v);
    v=msg.id &7;
    v<<=5;
    if( msg.rtr ==1)
      v|=0x10;
      v+=msg.dlen;
    can0w(11,v);
    for(i=0;i<msg.dlen; i++)
    {
      can0w((BYTE)(12+i),msg.data[i]);
    }
    can0w(1,1);
    while(1)
    {
      v=can0r(2);
      if( v & 0x40)
      {
        return( 0);
      }
      if( v &0x08)
        return( 1);
    }
  }
  else
  {
    v=msg.id>>3;
    can1w(10,v);
    v=msg.id &7;
    v<<=5;
    if( msg.rtr ==1)
      v|=0x10;
      v+=msg.dlen;
    can1w(11,v);
    for(i=0;i<msg.dlen; i++)
    {
      can1w((BYTE)(12+i),msg.data[i]);
    }
    can1w(1,1);
    while(1)
    {
      v=can1r(2);
      if( v & 0x40)
      {
        return( 0);
      }
      if( v &0x08)
        return( 1);
    }
  }
}
/*--------------------------------------------------------------------------*/
int  canReceiveMsg( BYTE port, MSG_STRUCT *msg_ptr)
{
BYTE i;

   if( port ==0)
   {
     if( gIrq0 == IRQ_POLLING)
     {
        if( can0r(2)&1)
        {
          rBuf0[0]=can0r(20);
          rBuf0[1]=can0r(21);
          for(i=0; i< (rBuf0[1]&0x0f); i++)
            rBuf0[i+2]=can0r((BYTE)(22+i));
          /* release receive buffer*/
          can0w(1,4);
          gRecFlag0=1;
        }
     }
      if( gRecFlag0 == 1)
      {
        gRecFlag0 = 0;
        msg_ptr->id=rBuf0[0];
        msg_ptr->id <<=3;
        msg_ptr->id |=(rBuf0[1] >>5);
        msg_ptr->rtr =(rBuf0[1]>>4)&1;
        msg_ptr->dlen =rBuf0[1]&0x0f;
        for(i=0;i< msg_ptr->dlen; i++)
          msg_ptr->data[i]=rBuf0[2+i];
        return( 1);
      }
      else
        return( 0);
   }
   else if(port ==1)
   {
     if( gIrq1 == IRQ_POLLING)
     {
        if( can1r(2)&1)
        {
          rBuf1[0]=can1r(20);
          rBuf1[1]=can1r(21);
          for(i=0; i< ( rBuf1[1]&0x0f); i++)
            rBuf1[i+2]=can1r((BYTE)(22+i));
          /* release receive buffer*/
          can1w(1,4);
          gRecFlag1=1;
        }
     }
      if( gRecFlag1 == 1)
      {
        gRecFlag1 = 0;
        msg_ptr->id=rBuf1[0];
        msg_ptr->id <<=3;
        msg_ptr->id |=(rBuf1[1] >>5);
        msg_ptr->rtr =(rBuf1[1]>>4)&1;
        msg_ptr->dlen =rBuf1[1]&0x0f;
        for(i=0;i< msg_ptr->dlen; i++)
          msg_ptr->data[i]=rBuf1[2+i];
        return( 1);
      }
      else
        return( 0);
   }
   else
     return( 0);
}
/*--------------------------------------------------------------------------*/
